%% This program replicates full sample results of the empirical illustation in Chudik, Kapetanios and Pesaran (2016, "A One-Covariate at a Time, Multiple Testing Approach to Variable Selection in High-Dimensional Linear Regression Models")
%   Sea readme.txt for further details.

clear; clc;

%% load and transform data
'Loading data...'

% load data
em=load('empir_m.txt'); % loads monthly variable set 1960M2:2008M11
    % em is 587 x 74
    %   data is organized in collumns (74 variables), with the first being the transformation code. 
    %   The first monthly observation is in the second row. (586 monthly time periods)
[Tm,nm]=size(em);
Tm=Tm-1;

eq=load('empir_q.txt'); % loads quarterly variable set 1960Q1 to 2008Q4
    % eq is 197 x 35
    %   data is organized in collumns (34 variables), with the first being the transformation code. 
    %   The first quarterly observation is in the second row. (196 quarterly time periods)    
[Tq,nq]=size(em);
Tq=Tq-1;
    
cpim=load('cpi.txt'); % loads cpi variable
    % cpi is 587 x 1
    %   the first row is the transformation code
gdp=load('gdp.txt'); % loads gdp variable
    % gdp is 197 x 1
    %   the first row is the transformation code
    
% data transformations
  % first sequentially sample the monthly variables into quarterly frequency (second month of each quarter)   
sq=[1:floor(Tm/3)+1]*3-2+1;    
emq=em(sq,:);
cpi=cpim(sq,1);

 % join the two quarterly datasets
data=[eq(2:end,:),emq];

 % transform data based on the transformation codes
 data = mtrans(data,[eq(1,:),em(1,:)]);
 gdp  = mtrans(gdp(2:end),gdp(1,1));
 cpi  = mtrans(cpi,cpim(1,1));
 
'Data is loaded and transformed.'

%% forecasting

'Computations have started and they might take few minutes...'

pmax=4; % max lag order
mmax=5; % maximum number of factors is set to 5
T=size(data,1); % time dimension of the sample
h=1; % forecasting horizon is set to 1. This code is written only for one-steap ahead forecasts.
wd=120; % define rolling window. Effective rolling window sample size for estimation is wd-pmax
ep=T-wd; % T-wd is used here for the full evaluation sample
         % T-wd-6 is used when evaluation period ends in 2007Q2. This is considered in the file 'program_OCMT_ea_SW_forecasting_S.m'.

nm=forecasts(pmax); % number of forecasting methods defined in forecasts.m
                % see forecasts.m for the description of individual methods
      
      
% initialize some variables
fcsts_g=NaN(ep,nm);
fcsts_c=NaN(ep,nm);

k=max(pmax)+mmax+size(data,2);
sel_allg=NaN(k,nm,ep);
sel_allc=NaN(k,nm,ep);

% compute individual forecasts

for t=1:ep
   % get subsample
   %[t,ep]
   s_start=t;
   s_end=t+wd-1;
   data_sample=data(s_start:s_end,:);
   gdp_sample=gdp(s_start:s_end,:);     
   cpi_sample=cpi(s_start:s_end,:);
   
   % compute forecasts
   [fcsts_g(t,:), sel_allg(:,:,t)]=forecasts(pmax, gdp_sample, data_sample, mmax);
   [fcsts_c(t,:), sel_allc(:,:,t)]=forecasts(pmax, cpi_sample, data_sample, mmax);
end

%% evaluate forecasting performance
 % compute relative RMSFE (vs AR(1) benchmark)
 benchmark=1; % the first method is the benchmark
 [rmsfe_g, relative_rmsfe_g, eg]=compute_rmsfe(gdp(wd+h:wd+ep+h-1,1), fcsts_g, benchmark);
 [rmsfe_c, relative_rmsfe_c, ec]=compute_rmsfe(cpi(wd+h:wd+ep+h-1,1), fcsts_c, benchmark);
 
 % compute DM statistics
 
 DMg=DMstats(eg);
 DMc=DMstats(ec);
 
'Computations are done.'

%% report location of results

disp('RMSFE results for AR(1), factor-augmented AR(1), Lasso, adaptive Lasso, and OCMT (p=0.1, 0.05, 0.01) are reported in the following 7x1 vectors:')
disp('  - rmsfe_g for output growth')
rmsfe_g
disp('  - rmsfe_c for inflation')
rmsfe_c
disp(' ')

disp('Diebold-Mariano (DM) test statistics are reported in the following 7x7 matrices:')
disp('  - DMg for output growth')
DMg
disp('  - DMc for inflation')
DMc
disp(' ')

disp('Identity of the selected regressors is saved in k x nm x ep array sel_allg for output growth and array sel_allc for inflation')
disp('  - k= 1 (for lagged dependent variable) + 109 (for the Stock & Watson loaded and transformed series, in the order they are loaded and transgformed) + 5 (PC1 to PC5 wth 5 being the max number of PCs considered')
disp('  - nm= 7 is the number of methods in the followingorder: AR(1), Factor-augmented AR(1), Lasso, Adaptie Lasso, and OCMT (p=0.01, 0.05, 0.1) ')
disp('  - ep is the number of evaluation periods')
disp('NaN value in sel_allg means variable was not selected. Value of 1 indicates variable was selected.')

%Selected regressors using Lasso
lsel_g=squeeze(sel_allg(:,3,:));
lsel_c=squeeze(sel_allc(:,3,:));

%Selected regressors using ALasso
alsel_g=squeeze(sel_allg(:,4,:));
alsel_c=squeeze(sel_allc(:,4,:));

%Selected regressors using OCMT
    % p=0.01
Osel1_g=squeeze(sel_allg(:,5,:));
Osel1_c=squeeze(sel_allc(:,5,:));
    % p=0.05
Osel2_g=squeeze(sel_allg(:,6,:));
Osel2_c=squeeze(sel_allc(:,6,:));
    % p=0.1
Osel3_g=squeeze(sel_allg(:,7,:));
Osel3_c=squeeze(sel_allc(:,7,:));



'saving selection outcomes in xls file...'
'   Lasso'
filename=[pwd,'\Pasted_results.xlsm'];
xlswrite(filename, squeeze(sel_allg(:,3,:)), 'SelVars_g_Lasso_pmax4', 'C2');
xlswrite(filename, squeeze(sel_allc(:,3,:)), 'SelVars_c_Lasso_pmax4', 'C2');


'A-Lasso'
xlswrite(filename, squeeze(sel_allg(:,4,:)), 'SelVars_g_ALasso_pmax4', 'C2');
xlswrite(filename, squeeze(sel_allc(:,4,:)), 'SelVars_c_ALasso_pmax4', 'C2');



'OCMT'

xlswrite(filename, squeeze(sel_allg(:,5,:)), 'SelVars_g_OCMT_p=0.01,pmax4', 'C2');
xlswrite(filename, squeeze(sel_allc(:,5,:)), 'SelVars_c_OCMT_p=0.01,pmax4', 'C2');

xlswrite(filename, squeeze(sel_allg(:,6,:)), 'SelVars_g_OCMT_p=0.05,pmax4', 'C2');
xlswrite(filename, squeeze(sel_allc(:,6,:)), 'SelVars_c_OCMT_p=0.05,pmax4', 'C2');

xlswrite(filename, squeeze(sel_allg(:,7,:)), 'SelVars_g_OCMT_p=0.1,pmax4', 'C2');
xlswrite(filename, squeeze(sel_allc(:,7,:)), 'SelVars_c_OCMT_p=0.1,pmax4', 'C2');

% save rmsfe
xlswrite(filename, rmsfe_g, 'RMSFE', 'I7');
xlswrite(filename, rmsfe_c, 'RMSFE', 'I15');

% xlswrite(filename, rmsfe_g, 'RMSFE', 'K7');
% xlswrite(filename, rmsfe_c, 'RMSFE', 'K15');

% save DM
xlswrite(filename, DMg, 'DM', 'C5');
xlswrite(filename, DMc, 'DM', 'K5');

% xlswrite(filename, DMg, 'DM', 'C15'); % pre-crisis
% xlswrite(filename, DMc, 'DM', 'K15'); % pre-crisis

'done'
